package com.nemrodsoftware.maven;

/*
 * Copyright 2001-2005 The Apache Software Foundation.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

import java.io.File;
import java.io.IOException;
import java.util.List;

import org.apache.commons.lang.StringUtils;
import org.apache.maven.plugin.AbstractMojo;
import org.apache.maven.plugin.MojoExecutionException;

import com.nemrodsoftware.INemrodClient;
import com.nemrodsoftware.NemrodClient;
import com.nemrodsoftware.model.Module;

/**
 * Goal which import resource file from a Nemrod instance
 *
 * @goal sync
 * 
 * @phase process-sources
 */
public class SyncMojo
    extends AbstractMojo
{
    /**
     * Resource folder
     * @parameter expression="${sync.workfolder}" default-value="${project.build.outputDirectory}"
     */
    private File workfolder;    
    
    /**
     * Login.
     * @parameter expression="${sync.apikey}" 
     * @required
     */
    private String apikey;

    /**
     * Url.
     * @parameter expression="${sync.url}"  default-value="http://live.nemrod-software.com"
     * @required
     */
    private String url;  
    
    /**
     * Project name.
     * @parameter expression="${sync.project}"
     * @required
     */
    private String projectName;        

    /**
     * the list of bundle to synchronize
     *
     * @parameter expression="${sync.bundles}"
     * @required
     */
    private List<String> bundleNames;

    /**
     * sync format
     *
     * @parameter expression="${sync.format}" default-value="java"
     * @required
     */
    private String format = "java";         

    public void execute()
        throws MojoExecutionException
    {
        try
        {
            INemrodClient client = new NemrodClient(url, apikey);
            
            getLog().info("Action : sync ");
            getLog().info("url="+url);
            getLog().info("project="+projectName);
            getLog().info("format="+format);
            
            for (String bundleName : bundleNames)
            {
                String moduleName;
                String modulePath;
                
                if (bundleName.contains("/"))
                {
                    moduleName = StringUtils.substringAfterLast(bundleName,"/");
                    modulePath = StringUtils.substringBeforeLast(bundleName,"/");
                }
                else
                {
                    moduleName= bundleName;
                    modulePath="";
                }                        
                File masterFile = new File(workfolder,bundleName+".properties");                
                
                getLog().info("");
                getLog().info("Synchronize default bundle ("+moduleName+ ")...");
                
                execute(client,masterFile,moduleName,modulePath);
            }            
        }
        catch (Exception e)
        {
            throw new MojoExecutionException("Unable to import resource bundles",e);
        }
    }
    
    public void execute(INemrodClient client, File masterFile, String moduleName, String modulePath) throws IOException
    {
        if (StringUtils.isEmpty(projectName))
                throw new IllegalArgumentException("Project name is not set. Please set nemrod.project property");
        if (StringUtils.isEmpty(format))
            throw new IllegalArgumentException("Format is not set. Please set nemrod.syncformat property");
        if (StringUtils.isEmpty(moduleName))
            throw new IllegalArgumentException("Module name is not set. Please set nemrod.module property");
        if (masterFile == null || !masterFile.exists())
            throw new IllegalArgumentException("Master file is invalid. Please set a valid path to the master file nemrod.sync.masterfile.path property");
        
        Module module = client.findModule(projectName,moduleName,modulePath);
        if (module==null)
        {
            throw new IllegalArgumentException("Module not found on nemrod instance for this project. Please check project configuration.");
        }
        
        client.sync(projectName, format, module, masterFile);
        
        getLog().info("Sync master file="+masterFile.getAbsolutePath()+" - OK");
        
    }
}
